#include "Map.h"
#include <Datei.h>
#include <InitDatei.h>
#include <KSGTDatei.h>
#include <Rahmen.h>
#include <TastaturEreignis.h>
#include <Zeit.h>
#include <Text.h>

// Inhalt der Map Klasse aus Map.h
// Konstruktor
Map::Map()
{
	schlange = new Array< Pos >();
	ziele = new Array< Pos >();
	feld = new LRahmen();
	feld->setFarbe( 0xFFFFFFFF );
	kam = new LRahmen();
	kam->setFarbe( 0xFF777777 );
	map = new LRahmen();
	map->setFarbe( 0xFFFFFFFF );
	map->setPosition( 10, 10 );
	map->setSize( 200, 200 );
	beendet = 1;
	move = 1;
	sr = 1;
	rend = 0;
	ref = 1;
}

// Destruktor
Map::~Map()
{
	speichern();
	schlange->release();
	ziele->release();
	feld->release();
	kam->release();
	map->release();
}

// nicht constant
void Map::reset()
{
	next = 0;
	beendet = 0;
	richtung = 0;
	addAnzahl = 0;
	score = 0;
	schlange->leeren();
	ziele->leeren();
	InitDatei *opd = new InitDatei( "data/Minigames/Snake/data/optionen.ini" );
	if( !opd->laden() )
		DateiPfadErstellen( "data/Minigames/Snake/data/optionen.ini" );
	breite = 80;
	if( opd->wertExistiert( "Breite" ) )
		breite = (int)*opd->zWert( "Breite" );
	else
		opd->addWert( "Breite", "80" );
	hhe = 50;
	if( opd->wertExistiert( "Hhe" ) )
		hhe = (int)*opd->zWert( "Hhe" );
	else
		opd->addWert( "Hhe", "50" );
	zAnzahl = 1;
	if( opd->wertExistiert( "Ziele" ) )
		zAnzahl = (int)*opd->zWert( "Ziele" );
	else
		opd->addWert( "Ziele", "1" );
	neuAnzahl = 10;
	if( opd->wertExistiert( "Anhngen" ) )
		neuAnzahl = (int)*opd->zWert( "Anhngen" );
	else
		opd->addWert( "Anhngen", "10" );
	geschwindigkeit = 10;
	if( opd->wertExistiert( "Geschwindigkeit" ) )
		geschwindigkeit = (int)*opd->zWert( "Geschwindigkeit" );
	else
		opd->addWert( "Geschwindigkeit", "10" );
	bool fortsetzen = 0;
	if( opd->wertExistiert( "Fortsetzen" ) )
		fortsetzen = (int)*opd->zWert( "Fortsetzen" ) != 0;
	else
		opd->addWert( "Fortsetzen", "0" );
	opd->speichern();
	opd->release();
	kamPos.x = 0;
	kamPos.y = 0;
	if( breite > 80 )
		kamPos.x = breite / 2 - 40;
	if( hhe > 50 )
		kamPos.y = hhe / 2 - 25;
	if( fortsetzen && DateiExistiert( "data/Minigames/Snake/data/game.save" ) )
	{
		Datei *save = new Datei();
		save->setDatei( "data/Minigames/Snake/data/game.save" );
		save->open( Datei::Style::lesen );
		int br = 0;
		int h = 0;
		save->lese( (char*)&br, 4 );
		save->lese( (char*)&h, 4 );
		if( br == breite && h == hhe )
		{
			save->lese( (char*)&score, 4 );
			save->lese( (char*)&richtung, 4 );
			int anz = 0;
			save->lese( (char*)&anz, 4 );
			for( int i = 0; i < anz; i++ )
			{
				Pos p;
				save->lese( (char*)&p.x, 2 );
				save->lese( (char*)&p.y, 2 );
				schlange->add( p );
			}
			save->lese( (char*)&anz, 4 );
			for( int i = 0; i < anz; i++ )
			{
				Pos p;
				save->lese( (char*)&p.x, 2 );
				save->lese( (char*)&p.y, 2 );
				ziele->add( p );
			}
			next = 1.0 / geschwindigkeit;
		}
		save->close();
		save->release();
	}
	if( !schlange->getEintragAnzahl() )
		schlange->add( Pos{ (short)( breite / 2 ), (short)( hhe / 2 ) } );
}

void Map::doMausEreignis( MausEreignis &me )
{

}

void Map::doTastaturEreignis( TastaturEreignis &te )
{
	if( !move )
		return;
	if( te.taste == T_Oben && richtung != 2 )
	{
		move = 0;
		richtung = 0;
	}
	if( te.taste == T_Rechts && richtung != 3 )
	{
		move = 0;
		richtung = 1;
	}
	if( te.taste == T_Unten && richtung != 0 )
	{
		move = 0;
		richtung = 2;
	}
	if( te.taste == T_Links && richtung != 1 )
	{
		move = 0;
		richtung = 3;
	}
}

bool Map::tick( double tickVal )
{
	if( beendet )
		return 0;
	if( sr )
		srand( (unsigned int)time( 0 ) );
	sr = 0;
	bool ret = rend;
	rend = 0;
	next -= tickVal;
	while( next < 0 && !beendet )
	{
		move = 1;
		ret = 1;
		next += 1.0 / geschwindigkeit;
		int nx = schlange->get( 0 ).x;
		int ny = schlange->get( 0 ).y;
		if( richtung == 0 )
			ny--;
		if( richtung == 1 )
			nx++;
		if( richtung == 2 )
			ny++;
		if( richtung == 3 )
			nx--;
		if( breite > 80 )
		{
			kamPos.x = nx - 40;
			if( kamPos.x < 0 )
				kamPos.x = 0;
			if( kamPos.x + 80 > breite )
				kamPos.x = breite - 80;
		}
		if( hhe > 50 )
		{
			kamPos.y = ny - 25;
			if( kamPos.y < 0 )
				kamPos.y = 0;
			if( kamPos.y + 50 > hhe )
				kamPos.y = hhe - 50;
		}
		int sAnz = schlange->getEintragAnzahl();
		for( int i = 0; i < sAnz; i++ )
			beendet |= nx == schlange->get( i ).x && ny == schlange->get( i ).y;
		beendet |= nx < 0 || nx >= breite;
		beendet |= ny < 0 || ny >= hhe;
		int zAnz = ziele->getEintragAnzahl();
		bool neuZ = !zAnz;
		for( int i = 0; i < zAnz; i++ )
			neuZ |= nx == ziele->get( i ).x && ny == ziele->get( i ).y;
		if( neuZ )
		{
			addAnzahl += neuAnzahl;
			ziele->leeren();
			for( int i = 0; i < zAnzahl; i++ )
				ziele->add( Pos{ (short)( rand() % breite ), (short)( rand() % hhe ) } );
			if( zAnz )
				score++;
		}
		if( !beendet )
		{
			schlange->add( Pos{ (short)nx, (short)ny }, 0 );
			if( !addAnzahl )
				schlange->remove( sAnz );
			else
				addAnzahl--;
		}
		else
		{
			KSGTDatei *stb = new KSGTDatei( "data/Minigames/Snake/data/score.ksgt" );
			if( !stb->laden() )
				DateiPfadErstellen( "data/Minigames/Snake/data/score.ksgt" );
			RCArray< Text > *zeile = new RCArray< Text >();
			Zeit *zeit = getZeit();
			zeile->add( zeit->getZeit( "y-m-d h:i:s" ) );
			zeit->release();
			Text *scoreT = new Text();
			scoreT->append( score );
			zeile->add( scoreT );
			Text *breiteT = new Text();
			breiteT->append( breite );
			zeile->add( breiteT );
			Text *hheT = new Text();
			hheT->append( hhe );
			zeile->add( hheT );
			Text *geschwindigkeitT = new Text();
			geschwindigkeitT->append( geschwindigkeit );
			zeile->add( geschwindigkeitT );
			Text *zAnzahlT = new Text();
			zAnzahlT->append( zAnzahl );
			zeile->add( zAnzahlT );
			Text *nAnzahlT = new Text();
			nAnzahlT->append( neuAnzahl );
			zeile->add( nAnzahlT );
			stb->addZeile( 7, zeile );
			zeile->release();
			stb->speichern();
			stb->release();
            DateiRemove( "data/Minigames/Snake/data/game.save" );
		}
	}
	return ret;
}

void Map::render( Bild &zRObj )
{
	int xStart = 0;
	int yStart = 0;
	if( breite < 80 )
		xStart = 400 - breite * 5;
	else
		xStart -= kamPos.x * 10;
	if( hhe < 50 )
		yStart = 250 - hhe * 5;
	else
		yStart -= kamPos.y * 10;
	feld->setPosition( xStart, yStart );
	feld->setSize( breite * 10, hhe * 10 );
	feld->render( zRObj );
	bool rMap = breite > 80 || hhe > 50;
	int sLn = schlange->getEintragAnzahl();
	for( int i = 0; i < sLn; i++ )
		zRObj.fillRegion( xStart + schlange->get( i ).x * 10, yStart + schlange->get( i ).y * 10, 9, 9, 0xFFFFFFFF );
	int zAnz = ziele->getEintragAnzahl();
	for( int i = 0; i < zAnz; i++ )
		zRObj.fillRegion( xStart + ziele->get( i ).x * 10, yStart + ziele->get( i ).y * 10, 9, 9, 0xFF00FF00 );
	if( rMap )
	{
		const Punkt &dOff = zRObj.getDrawOff();
		map->render( zRObj );
		for( int i = 0; i < sLn; i++ )
			zRObj.setPixelDP( 10 + ( 200 * schlange->get( i ).x ) / breite + dOff.x, 10 + ( 200 * schlange->get( i ).y ) / hhe + dOff.y, 0xFFFFFFFF );
		for( int i = 0; i < zAnz; i++ )
			zRObj.setPixelDP( 10 + ( 200 * ziele->get( i ).x ) / breite + dOff.x, 10 + ( 200 * ziele->get( i ).y ) / hhe + dOff.y, 0xFF00FF00 );
		kam->setPosition( 10 + ( 200 * kamPos.x ) / breite, 10 + ( 200 * kamPos.y ) / hhe );
		kam->setSize( 16000 / breite, ( 10000 ) / hhe );
		if( kam->getBreite() > 200 )
			kam->setSize( 200, kam->getHeight() );
		if( kam->getHeight() > 200 )
			kam->setSize( kam->getBreite(), 200 );
		kam->render( zRObj );
	}
}

// constant
void Map::speichern() const
{
	if( !beendet )
	{
		Datei *d = new Datei();
		d->setDatei( "data/Minigames/Snake/data/game.save" );
		d->erstellen();
		d->open( Datei::Style::schreiben );
		d->schreibe( (char*)&breite, 4 );
		d->schreibe( (char*)&hhe, 4 );
		d->schreibe( (char*)&score, 4 );
		d->schreibe( (char*)&richtung, 4 );
		int anz = schlange->getEintragAnzahl();
		d->schreibe( (char*)&anz, 4 );
		for( int i = 0; i < anz; i++ )
		{
			short p = schlange->get( i ).x;
			d->schreibe( (char*)&p, 2 );
			p = schlange->get( i ).y;
			d->schreibe( (char*)&p, 2 );
		}
		anz = ziele->getEintragAnzahl();
		d->schreibe( (char*)&anz, 4 );
		for( int i = 0; i < anz; i++ )
		{
			short p = ziele->get( i ).x;
			d->schreibe( (char*)&p, 2 );
			p = ziele->get( i ).y;
			d->schreibe( (char*)&p, 2 );
		}
		d->close();
		d->release();
	}
	else
		DateiRemove( "data/Minigames/Snake/data/game.save" );
}

int Map::getScore() const
{
	return score;
}

bool Map::istBeendet() const
{
	return beendet;
}

// Reference Counting
Map *Map::getThis()
{
	ref++;
	return this;
}

Map *Map::release()
{
	ref--;
	if( !ref )
		delete this;
	return 0;
}