#ifndef Men_H
#define Men_H

#include <Knopf.h>
#include <Tabelle.h>
#include <Diagramm.h>
#include <AuswahlBox.h>
#include "Map.h"

using namespace Framework;

class MenVerloren
{
private:
	LRahmen *ram;
	TextFeld *verloren;
	Knopf *neu;
	Knopf *beenden;
	int ref;

public:
	// Kontruktor
	MenVerloren( Schrift *zSchrift );
	// Destruktor
	~MenVerloren();
	// nicht constant
	int doMausEreignis( MausEreignis &me );
	bool tick( double tickVal );
	void render( Bild &zRObj );
	// Reference Counting
	MenVerloren *getThis();
	MenVerloren *release();
};

class MenSpiel
{
private:
	TextFeld *scoreT;
	Knopf *beenden;
	Map *map;
	MenVerloren *verloren;
	bool beendet;
	int ref;

public:
	// Konstruktor
	MenSpiel( Schrift *zSchrift, Bildschirm *zScreen );
	// Destruktor
	~MenSpiel();
	// nicht constant
	void reset();
	void doMausEreignis( MausEreignis &me );
	void doTastaturEreignis( TastaturEreignis &te );
	bool tick( double tickVal );
	void render( Bild &zRObj );
	// constant
	bool istBeendet() const;
	// Reference Counting
	MenSpiel *getThis();
	MenSpiel *release();
};

class MenStatistik
{
private:
	TextFeld *ansichtT;
	AuswahlBox *ansicht;
	KontrollKnopf *optionen;
	ObjTabelle *gesammtT;
	ObjTabelle *optionenT;
	LDiag *gesammtD;
	LDiag *optionenD;
	Knopf *releasen;
	Knopf *zurck;
	Schrift *schrift;
	bool beendet;
	int ref = 1;

public:
	// Konstruktor
	MenStatistik( Schrift *zSchrift, Bildschirm *zScreen );
	// Destruktor
	~MenStatistik();
	// nicht constant
	void reset();
	void doMausEreignis( MausEreignis &me );
	bool tick( double tickVal );
	void render( Bild &zRObj );
	// constant
	bool istBeendet() const;
	// Reference Counting
	MenStatistik *getThis();
	MenStatistik *release();
};

class MenOptionen
{
private:
	TextFeld *breiteT;
	TextFeld *breite;
	TextFeld *hheT;
	TextFeld *hhe;
	TextFeld *zAnzahlT;
	TextFeld *zAnzahl;
	TextFeld *nAnzahlT;
	TextFeld *nAnzahl;
	TextFeld *tempoT;
	TextFeld *tempo;
	KontrollKnopf *fortsetzen;
	Knopf *abbrechen;
	Knopf *ok;
	bool beendet;
	int ref;

public:
	// Konstruktor
	MenOptionen( Schrift *zSchrift, Bildschirm *zScreen );
	// Destruktor
	~MenOptionen();
	// nicht constant
	void reset();
	void doMausEreignis( MausEreignis &me );
	void doTastaturEreignis( TastaturEreignis &te );
	bool tick( double tickVal );
	void render( Bild &zRObj );
	// constant
	bool istBeendet() const;
	// Reference Counting
	MenOptionen *getThis();
	MenOptionen *release();
};

class Men
{
private:
	Knopf *spielen;
	MenSpiel *spielenF;
	Knopf *optionen;
	MenOptionen *optionenF;
	Knopf *statistik;
	MenStatistik *statistikF;
	Knopf *beenden;
	Bild *hintergrund;
	LRahmen *ram;
	bool beendet;
	int status;
	int ref;

public:
	// Konstruktor
	Men( Schrift *zSchrift, Bildschirm *zScreen );
	// Destruktor
	~Men();
	// nicht constant#
	void reset();
	void doMausEreignis( MausEreignis &me );
	void doTastaturEreignis( TastaturEreignis &te );
	bool tick( double z );
	void render( Bild &zRObj );
	// constant
	bool istBeendet() const;
	// Reference Counting
	Men *getThis();
	Men *release();
};

#endif